/* core01.c

*/

#include <stdio.h>
#include <windows.h>

#include "types.h"
#include "md5.h"

/* External Fucntions */
void undo64to56(ord32 *out, ord32 *in, ord8 *buffer); 

/* Internal Functions */
static int MD5Transform( ord8 * block);

/* Internal Globals Varibles */
static ord32 fasta;


int
searchcore01( ord32 totalLen, ord32 suffixLen, ord32 searchLen, ord32 listLen,
	     ord32 *hash, ord32 *startPoint, ord8 *suffix, ord8 *list)

{
	ord32 i, tail, pos;
	ord32 fastb, fastc, fastd;
	ord32 *x;
	ord32 tested;


	ord8 buffer[60];
	ord32 workhash[4];
	ord32 workpoint[7];

	int timea, timeb;

	timea = GetTickCount();

	x = (ord32*)buffer;


	if((totalLen < 1) || (totalLen > 55)) {
		// password too long
		return 0;
	}

	// work out tail value;
	tail = totalLen << 3;

	// clear buffer
	memset(buffer, 0, 60);

	// copy tail into buffer	 	
	memcpy(buffer+56,&tail,4);	

	//copy prefix into buffer
	memcpy(buffer+searchLen, suffix, suffixLen);

	// set start of PADDING
	buffer[totalLen] = 0x80;

	//copy start value into buffer
	for(i=0; i<searchLen; i++ ) {
		buffer[i] = list[ startPoint[i] ];
	}

	//make locale copy of startPoint
	memcpy(workpoint,startPoint,sizeof(ord32)*7);

	undo64to56(workhash, hash, buffer);

	/* do work on block */

	tested=0;
	pos=4;  

	while( 1 ) 
	{
		if( pos > 3) 
		{
			fasta = workhash[0]; 
			fastb = workhash[1]; 
			fastc = workhash[2]; 
			fastd = workhash[3]; 

			fastb -= x[ 1];
			RII (fastc, fastd, fasta, fastb, x[10], S43, 0xffeff47d); /* 55 */
			RII (fastd, fasta, fastb, fastc, x[ 3], S42, 0x8f0ccc92); /* 54 */
			RII (fasta, fastb, fastc, fastd, x[12], S41, 0x655b59c3); /* 53 */
			RII (fastb, fastc, fastd, fasta, x[ 5], S44, 0xfc93a039); /* 52 */
			RII (fastc, fastd, fasta, fastb, x[14],  S43, 0xab9423a7); /* 51 */ 
			RII (fastd, fasta, fastb, fastc, x[ 7], S42, 0x432aff97); /* 50 */
			RII0 (fasta, fastb, fastc, fastd,       S41, 0xf4292244); /* 49 */
		}

		tested++;
		//printf("%s \n",buffer);

	    
		if( MD5Transform( buffer )) 
		{
			float rate;

			timeb = GetTickCount(); 
			timea = timeb-timea;
			rate = (float)tested / ((float)timea/1000.0f);

			printf("Tested: %u keys, Time: %u Rate: %.0f Status: Found\n",
				tested,timea,rate);

			return 1;
		} 
        
		// increase search string.
		pos = 0;
		while(2) {
			workpoint[pos]++;

			if(workpoint[pos] < listLen) {
				break;
			}

			workpoint[pos] = 0;


			if( pos == searchLen-1 ) {
				float rate;

				timeb = GetTickCount(); 
				timea = timeb-timea;
				rate = (float)tested / ((float)timea/1000.0f);

				printf("Tested: %u keys, Time: %u Rate: %.0f Status: Not Found\n",
					tested,timea,rate);

				return 0;

			}
			pos++;

		} // while (2)

		// refill buffer
		for(i=0; i <= pos; i++) { 
			buffer[i] = list[ workpoint[i] ]; 
		} 

	} // while (1)
  

	return 0;
}

static int
MD5Transform( ord8 * block)
{
	ord32	a, b, c, d;
	ord32* x = (ord32 *)block;

	b = 0xefcdab89;
	c = 0x98badcfe;
	d = 0x10325476; 

	a = 0xd76aa477 + x[ 0];
	ROTATE_LEFT (a, S11); 
	a += b; 
	//FF (a, b, c, d, x[ 0], S11, 0xd76aa478); /* 1 */  

	FF (d, a, b, c, x[ 1], S12, 0xe8c7b756); /* 2 */

	FF (c, d, a, b, x[ 2], S13, 0x242070db); /* 3 */
	FF (b, c, d, a, x[ 3], S14, 0xc1bdceee); /* 4 */
	FF (a, b, c, d, x[ 4], S11, 0xf57c0faf); /* 5 */
	FF (d, a, b, c, x[ 5], S12, 0x4787c62a); /* 6 */
	FF (c, d, a, b, x[ 6], S13, 0xa8304613); /* 7 */
	FF (b, c, d, a, x[ 7], S14, 0xfd469501); /* 8 */
	FF (a, b, c, d, x[ 8], S11, 0x698098d8); /* 9 */
	FF (d, a, b, c, x[ 9], S12, 0x8b44f7af); /* 10 */

	FF (c, d, a, b, x[10], S13, 0xffff5bb1); /* 11 */
	FF (b, c, d, a, x[11], S14, 0x895cd7be); /* 12 */
	FF (a, b, c, d, x[12], S11, 0x6b901122); /* 13 */
	FF (d, a, b, c, x[13], S12, 0xfd987193); /* 14 */

	FF (c, d, a, b, x[14],  S13, 0xa679438e); /* 15 */
	FF0 (b, c, d, a,       S14, 0x49b40821); /* 16 */

	/* Round 2 */
	GG (a, b, c, d, x[ 1], S21, 0xf61e2562); /* 17 */
	GG (d, a, b, c, x[ 6], S22, 0xc040b340); /* 18 */
	GG (c, d, a, b, x[11], S23, 0x265e5a51); /* 19 */
	GG (b, c, d, a, x[ 0], S24, 0xe9b6c7aa); /* 20 */
	GG (a, b, c, d, x[ 5], S21, 0xd62f105d); /* 21 */
	GG (d, a, b, c, x[10], S22, 0x2441453); /* 22 */
	GG0 (c, d, a, b,       S23, 0xd8a1e681); /* 23 x[15] */
	GG (b, c, d, a, x[ 4], S24, 0xe7d3fbc8); /* 24 */
	GG (a, b, c, d, x[ 9], S21, 0x21e1cde6); /* 25 */
	GG (d, a, b, c, x[14], S22, 0xc33707d6); /* 26 */
	GG (c, d, a, b, x[ 3], S23, 0xf4d50d87); /* 27 */
	GG (b, c, d, a, x[ 8], S24, 0x455a14ed); /* 28 */
	GG (a, b, c, d, x[13], S21, 0xa9e3e905); /* 29 */
	GG (d, a, b, c, x[ 2], S22, 0xfcefa3f8); /* 30 */
	GG (c, d, a, b, x[ 7], S23, 0x676f02d9); /* 31 */
	GG (b, c, d, a, x[12], S24, 0x8d2a4c8a); /* 32 */

	/* Round 3 */
	HH (a, b, c, d, x[ 5], S31, 0xfffa3942); /* 33 */
	HH (d, a, b, c, x[ 8], S32, 0x8771f681); /* 34 */
	HH (c, d, a, b, x[11], S33, 0x6d9d6122); /* 35 */
	HH (b, c, d, a, x[14], S34, 0xfde5380c); /* 36 */
	HH (a, b, c, d, x[ 1], S31, 0xa4beea44); /* 37 */
	HH (d, a, b, c, x[ 4], S32, 0x4bdecfa9); /* 38 */
	HH (c, d, a, b, x[ 7], S33, 0xf6bb4b60); /* 39 */
	HH (b, c, d, a, x[10], S34, 0xbebfbc70); /* 40 */
	HH (a, b, c, d, x[13], S31, 0x289b7ec6); /* 41 */
	HH (d, a, b, c, x[ 0], S32, 0xeaa127fa); /* 42 */
	HH (c, d, a, b, x[ 3], S33, 0xd4ef3085); /* 43 */
	HH (b, c, d, a, x[ 6], S34, 0x4881d05); /* 44 */
	HH (a, b, c, d, x[ 9], S31, 0xd9d4d039); /* 45 */

	a += x[0];

	if( fasta == a )
		return 1;

	return 0;
}
