/* core05.c

*/

#include <stdio.h>
#include <windows.h>

#include "types.h"
#include "md5.h"

/* External Fucntions */
void undo64to56(ord32 *out, ord32 *in, ord8 *buffer); 

/* Internal Functions */
static int MD5Transform( ord8 * block);
static ord32 doWork(ord32 searchLen, ord32 listLen, ord32 *letters, ord8 *list,ord32 *hash, ord64 *count); 


/* Internal Globals Varibles */
static ord32 fasta;
static ord32 *x;

int
searchcore06( ord32 totalLen, ord32 suffixLen, ord32 searchLen, ord32 listLen,
	     ord32 *hash, ord32 *startPoint, ord8 *suffix, ord8 *list)

{
	ord32 i, tail;
	ord64 tested;
	ord32 ret;

	ord8 buffer[60];
	ord32 workhash[4];
	ord32 workpoint[7];
 
	int timea, timeb;
	float rate;

    timea = GetTickCount();

	x = (ord32*)buffer;


	if((totalLen < 1) || (totalLen > 55)) {
		// password too long
		return 0;
	}

	// work out tail value;
	tail = totalLen << 3;
  
	// clear buffer
	memset(buffer, 0, 60);

	// copy tail into buffer	 	
	memcpy(buffer+56,&tail,4);	

	//copy prefix into buffer
	memcpy(buffer+searchLen, suffix, suffixLen);
  
	// set start of PADDING
	buffer[totalLen] = 0x80;

	//copy start value into buffer
	for(i=0; i<searchLen; i++ ) {
		buffer[i] = list[ startPoint[i] ];
	}

	//make locale copy of startPoint
	memcpy(workpoint,startPoint,sizeof(ord32)*7);

	undo64to56(workhash, hash, buffer);

	/* do work on block */

	ret = doWork(searchLen, listLen, workpoint, list, workhash, &tested);

	timeb = GetTickCount(); 
	timea = timeb-timea;
	rate = (float)tested / ((float)timea/1000.0f);

	printf("Tested: %llu keys, Time: %ld Rate: %.0f Status: %s\n",
			tested,timea,rate,(ret)?"Found":"Not Found");
 
  return 0;
}


static ord32
doWork(ord32 searchLen, ord32 listLen, ord32 *letters, ord8 *list,ord32 *hash, ord64 *count) 
{
	ord32 b, c, d;
	ord32 pos, i;
	ord8 *buffer = (ord8*)x;

	*count=0;
	pos=4;  
 
	while( 1 ) {
		if( pos > 3) {
			fasta = hash[0]; 
 			b = hash[1]; 
 			c = hash[2]; 
 			d = hash[3]; 

			b -= x[ 1];
			RII  (c, d, fasta, b, x[10], S43, 0xffeff47d); /* 55 */
			RII  (d, fasta, b, c, x[ 3], S42, 0x8f0ccc92); /* 54 */
			RII  (fasta, b, c, d, x[12], S41, 0x655b59c3); /* 53 */
			RII  (b, c, d, fasta, x[ 5], S44, 0xfc93a039); /* 52 */
			RII  (c, d, fasta, b, x[14], S43, 0xab9423a7); /* 51 */ 
			RII  (d, fasta, b, c, x[ 7], S42, 0x432aff97); /* 50 */
			RII0 (fasta, b, c, d,        S41, 0xf4292244); /* 49 */
		}

	*count+=1;
 
	    
    if( MD5Transform( buffer )) {
		return 1;
    } 
        
    // increase search string.
    pos = 0;
    while(2) {
		letters[pos]++;
      
		if(letters[pos] < listLen) {
			break;
		}
      
		letters[pos] = 0;
    	
  
		if( pos == searchLen-1 ) {

			return 0; // search space over.
		}
		pos++;
      
    } // while (2)
    
    // refill buffer
    for(i=0; i <= pos; i++) { 
	     buffer[i] = list[ letters[i] ]; 
    } 
    
  } // while (1)

}


#ifndef USE_ARRAYS

static const ord32 setup[4] = { 0x67452301, 0xefcdab89, 0x98badcfe, 0x10325476 }; 
#define startA setup[0]
#define startB setup[1]
#define startC setup[2]
#define startD setup[3]

#else

#define startA 0x67452301
#define startB 0xefcdab89
#define startC 0x98badcfe
#define startD 0x10325476


#endif USE_ARRAYS

#ifdef USE_AC_ARRAY

static const ord32 ac[45] = {
0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee,
0xf57c0faf, 0x4787c62a, 0xa8304613, 0xfd469501,
0x698098d8, 0x8b44f7af, 0xffff5bb1, 0x895cd7be,
0x6b901122, 0xfd987193, 0xa679438e, 0x49b40821,
0xf61e2562, 0xc040b340, 0x265e5a51, 0xe9b6c7aa,
0xd62f105d, 0x02441453, 0xd8a1e681, 0xe7d3fbc8,
0x21e1cde6, 0xc33707d6, 0xf4d50d87, 0x455a14ed,
0xa9e3e905, 0xfcefa3f8, 0x676f02d9, 0x8d2a4c8a,
0xfffa3942, 0x8771f681, 0x6d9d6122, 0xfde5380c,
0xa4beea44, 0x4bdecfa9, 0xf6bb4b60, 0xbebfbc70,
0x289b7ec6, 0xeaa127fa, 0xd4ef3085, 0x04881d05,
0xd9d4d039,
};

#define AC01 ac[0]
#define AC02 ac[1]
#define AC03 ac[2]
#define AC04 ac[3]
#define AC05 ac[4]
#define AC06 ac[5]
#define AC07 ac[6]
#define AC08 ac[7]
#define AC09 ac[8]
#define AC10 ac[9]
#define AC11 ac[10]
#define AC12 ac[11]
#define AC13 ac[12]
#define AC14 ac[13]
#define AC15 ac[14]
#define AC16 ac[15]
#define AC17 ac[16]
#define AC18 ac[17]
#define AC19 ac[18]
#define AC20 ac[19]
#define AC21 ac[20]
#define AC22 ac[21]
#define AC23 ac[22]
#define AC24 ac[23]
#define AC25 ac[24]
#define AC26 ac[25]
#define AC27 ac[26]
#define AC28 ac[27]
#define AC29 ac[28]
#define AC30 ac[29]
#define AC31 ac[30]
#define AC32 ac[31]
#define AC33 ac[32]
#define AC34 ac[33]
#define AC35 ac[34]
#define AC36 ac[35]
#define AC37 ac[36]
#define AC38 ac[37]
#define AC39 ac[38]
#define AC40 ac[39]
#define AC41 ac[40]
#define AC42 ac[41]
#define AC43 ac[42]
#define AC44 ac[43]
#define AC45 ac[44]

#else

#define AC01	0xd76aa478 /* 1 */  
#define AC02	0xe8c7b756 /* 2 */
#define AC03	0x242070db /* 3 */
#define AC04	0xc1bdceee /* 4 */
#define AC05	0xf57c0faf /* 5 */
#define AC06	0x4787c62a /* 6 */
#define AC07	0xa8304613 /* 7 */
#define AC08	0xfd469501 /* 8 */
#define AC09	0x698098d8 /* 9 */
#define AC10	0x8b44f7af /* 10 */
#define AC11	0xffff5bb1 /* 11 */
#define AC12	0x895cd7be /* 12 */
#define AC13	0x6b901122 /* 13 */
#define AC14	0xfd987193 /* 14 */
#define AC15	0xa679438e /* 15 */
#define AC16	0x49b40821 /* 16 */
#define AC17	0xf61e2562 /* 17 */
#define AC18	0xc040b340 /* 18 */
#define AC19	0x265e5a51 /* 19 */
#define AC20	0xe9b6c7aa /* 20 */
#define AC21	0xd62f105d /* 21 */
#define AC22	0x02441453 /* 22 */
#define AC23	0xd8a1e681 /* 23 */
#define AC24	0xe7d3fbc8 /* 24 */
#define AC25	0x21e1cde6 /* 25 */
#define AC26	0xc33707d6 /* 26 */
#define AC27	0xf4d50d87 /* 27 */
#define AC28	0x455a14ed /* 28 */
#define AC29	0xa9e3e905 /* 29 */
#define AC30	0xfcefa3f8 /* 30 */
#define AC31	0x676f02d9 /* 31 */
#define AC32	0x8d2a4c8a /* 32 */
#define AC33	0xfffa3942 /* 33 */
#define AC34	0x8771f681 /* 34 */
#define AC35	0x6d9d6122 /* 35 */
#define AC36	0xfde5380c /* 36 */
#define AC37	0xa4beea44 /* 37 */
#define AC38	0x4bdecfa9 /* 38 */
#define AC39	0xf6bb4b60 /* 39 */
#define AC40	0xbebfbc70 /* 40 */
#define AC41	0x289b7ec6 /* 41 */
#define AC42	0xeaa127fa /* 42 */
#define AC43	0xd4ef3085 /* 43 */
#define AC44	0x04881d05 /* 44 */
#define AC45	0xd9d4d039 /* 45 */

#endif USE_AC_ARRAY

#define FF1(v, x, s, ac ) FF(v[0],v[1],v[2],v[3],x,s,ac)
#define FF2(v, x, s, ac ) FF(v[3],v[0],v[1],v[2],x,s,ac)
#define FF3(v, x, s, ac ) FF(v[2],v[3],v[0],v[1],x,s,ac)
#define FF4(v, x, s, ac ) FF(v[1],v[2],v[3],v[0],x,s,ac)

#define GG1(v, x, s, ac ) GG(v[0],v[1],v[2],v[3],x,s,ac)
#define GG2(v, x, s, ac ) GG(v[3],v[0],v[1],v[2],x,s,ac)
#define GG3(v, x, s, ac ) GG(v[2],v[3],v[0],v[1],x,s,ac)
#define GG4(v, x, s, ac ) GG(v[1],v[2],v[3],v[0],x,s,ac)

#define HH1(v, x, s, ac ) HH(v[0],v[1],v[2],v[3],x,s,ac)
#define HH2(v, x, s, ac ) HH(v[3],v[0],v[1],v[2],x,s,ac)
#define HH3(v, x, s, ac ) HH(v[2],v[3],v[0],v[1],x,s,ac)
#define HH4(v, x, s, ac ) HH(v[1],v[2],v[3],v[0],x,s,ac)


static int
MD5Transform( ord8 * block)
{
  ord32 v[4];

  v[0] = startA;
  v[1] = startB;
  v[2] = startC;
  v[3] = startD; 
  v[4] = startA;
  v[5] = startB;
  v[6] = startC;
  v[7] = startD; 

  FF1 (v, x[ 0], S11, AC01 ); /* 1 */
  FF2 (v, x[ 1], S12, AC02 ); /* 2 */
  FF3 (v, x[ 2], S13, AC03 ); /* 3 */
  FF4 (v, x[ 3], S14, AC04 ); /* 4 */
  FF1 (v, x[ 4], S11, AC05 ); /* 5 */
  FF2 (v, x[ 5], S12, AC06 ); /* 6 */
  FF3 (v, x[ 6], S13, AC07 ); /* 7 */
  FF4 (v, x[ 7], S14, AC08 ); /* 8 */
  FF1 (v, x[ 8], S11, AC09 ); /* 9 */
  FF2 (v, x[ 9], S12, AC10 ); /* 10 */
  FF3 (v, x[10], S13, AC11 ); /* 11 */
  FF4 (v, x[11], S14, AC12 ); /* 12 */
  FF1 (v, x[12], S11, AC13 ); /* 13 */
  FF2 (v, x[13], S12, AC14 ); /* 14 */
  FF3 (v, x[14], S13, AC15 ); /* 15 */
  FF4 (v, 0,       S14, AC16 ); /* 16 */

  /* Round 2 */
  GG1 (v, x[ 1], S21, AC17 ); /* 17 */
  GG2 (v, x[ 6], S22, AC18 ); /* 18 */
  GG3 (v, x[11], S23, AC19 ); /* 19 */
  GG4 (v, x[ 0], S24, AC20 ); /* 20 */
  GG1 (v, x[ 5], S21, AC21 ); /* 21 */
  GG2 (v, x[10], S22, AC22 ); /* 22 */
  GG3 (v, 0,     S23, AC23 ); /* 23 x[15] */
  GG4 (v, x[ 4], S24, AC24 ); /* 24 */
  GG1 (v, x[ 9], S21, AC25 ); /* 25 */
  GG2 (v, x[14], S22, AC26 ); /* 26 */
  GG3 (v, x[ 3], S23, AC27 ); /* 27 */
  GG4 (v, x[ 8], S24, AC28 ); /* 28 */
  GG1 (v, x[13], S21, AC29 ); /* 29 */
  GG2 (v, x[ 2], S22, AC30 ); /* 30 */
  GG3 (v, x[ 7], S23, AC31 ); /* 31 */
  GG4 (v, x[12], S24, AC32 ); /* 32 */

  /* Round 3 */
  HH1 (v, x[ 5], S31, AC33 ); /* 33 */
  HH2 (v, x[ 8], S32, AC34 ); /* 34 */
  HH3 (v, x[11], S33, AC35 ); /* 35 */
  HH4 (v, x[14], S34, AC36 ); /* 36 */
  HH1 (v, x[ 1], S31, AC37 ); /* 37 */
  HH2 (v, x[ 4], S32, AC38 ); /* 38 */
  HH3 (v, x[ 7], S33, AC39 ); /* 39 */
  HH4 (v, x[10], S34, AC40 ); /* 40 */
  HH1 (v, x[13], S31, AC41 ); /* 41 */
  HH2 (v, x[ 0], S32, AC42 ); /* 42 */
  HH3 (v, x[ 3], S33, AC43 ); /* 43 */
  HH4 (v, x[ 6], S34, AC44 ); /* 44 */
  HH1 (v, x[ 9], S31, AC45 ); /* 45 */
  
  v[0] += x[0];
	
  if( fasta == v[0] )
	  return 1;

  return 0;
}




