/* core08.c
 * a double barrel decoder for 7 byte or less phrases 
*/

#include <stdio.h>
#include <windows.h>

#include "types.h"
#include "md5.h"

/* External Fucntions */
void undo64to56L7(ord32 *out, ord32 *in); 

/* Internal Functions */
static int MD5TransformL7( const ord32 x0a, const ord32 x0b, const ord32 x1, const ord32 x14 );
static int searchcore08Inner( ord32 totalLen, ord32 suffixLen, ord32 searchLen, ord32 listLen,
	     ord32 *hash, ord32 *startPoint, ord8 *suffix, ord8 *list);
	
/* Internal Globals Varibles */
static ord32 fasta;
//static ord32 tested;

typedef union {
	ord32 dword;
	ord8  byte[4];
} ord32_8;
	
int
searchcore08( ord32 totalLen, ord32 suffixLen, ord32 searchLen, ord32 listLen,
	     ord32 *hash, ord32 *startPoint, ord8 *suffix, ord8 *list)
{
	ord32 timea, timeb;
	int retval;
	float rate;
	ord64 tested;

/*	Check the test lenght is not 0, greater than this cores max, and that there is
	no suffix.
 */
	if( (totalLen < 1) || (totalLen > 7) || (suffixLen != 0) )
	{
		return 0;
	}

	tested = 0;

	timea = GetTickCount();

	retval = searchcore08Inner( totalLen, suffixLen, searchLen, listLen, hash, startPoint, suffix, list);  

	timeb = GetTickCount();


	timea = timeb-timea;
	rate = (float)tested / ((float)timea/1000.0f);

	printf("Tested: %llu keys, Time: %ld Rate: %.0f Status: %s\n",
			tested, timea, rate, (retval)?"Found":"Not Found");

	return retval;
}

static int		 
searchcore08Inner( ord32 totalLen, ord32 suffixLen, ord32 searchLen, ord32 listLen,
	     ord32 *hash, ord32 *startPoint, ord8 *suffix, ord8 *list)
{
	ord32 tail, pos;
	ord32 fastb,fastc,fastd;	
    ord32 tested;
	ord32_8 x0a, x0b, x1;

	ord32 workhash[4];
	ord32 workpoint[9];

//	work out tail value;
	tail = totalLen << 3;
    	
	x0a.byte[0] = list[ startPoint[0] ];
	x0a.byte[1] = list[ startPoint[1] ];
	x0a.byte[2] = list[ startPoint[2] ];
	x0a.byte[3] = list[ startPoint[3] ];
	
	x0b.dword = x0a.dword;
	x0b.byte[0] = list[ startPoint[0]+1 ];
	
//	make locale copy of startPoint
	memcpy(workpoint,startPoint,sizeof(ord32)*9);

	undo64to56L7(workhash, hash);

/*	do work on block */

	tested=0;
/*	set pos to be 4, so that 55 to 49 are undone for the first loop, also so x1 is loaded.
 */
	pos=4;  

	while( 1 ) 
	{
/*		If one of the bytes in the second dword (x1) have changed,
		reload x1, and then undo 55 to 49.
 */
		if( pos > 3)
		{
			x1.byte[0] = list[ workpoint[4] ];
			x1.byte[1] = list[ workpoint[5] ];
			x1.byte[2] = list[ workpoint[6] ];
			x1.byte[3] = list[ workpoint[7] ];

			fasta = workhash[0]; 
			fastb = workhash[1]; 
			fastc = workhash[2]; 
			fastd = workhash[3]; 

			fastb -= x1.dword;
			RII0 (fastc, fastd, fasta, fastb,		S43, 0xffeff47d); /* 55 */
			RII0 (fastd, fasta, fastb, fastc,		S42, 0x8f0ccc92); /* 54 */
			RII0 (fasta, fastb, fastc, fastd,		S41, 0x655b59c3); /* 53 */
			RII0 (fastb, fastc, fastd, fasta,		S44, 0xfc93a039); /* 52 */
			RII (fastc, fastd, fasta, fastb, tail,	S43, 0xab9423a7); /* 51 */ 
			RII0 (fastd, fasta, fastb, fastc,		S42, 0x432aff97); /* 50 */
			RII0 (fasta, fastb, fastc, fastd,		S41, 0xf4292244); /* 49 */   
		}

		tested+=2;

		if( MD5TransformL7( x0a.dword, x0b.dword, x1.dword, tail ) ) 
		{
			return 1;
		} 
        
//		increase search string.

		pos = 0;	
		workpoint[0] += 2;

		if(workpoint[0] < listLen)
		{
			goto refil;
		}

		while(2) 
		{
			workpoint[pos]++;

			if(workpoint[pos] < listLen)
			{
				break;
			}
      
			workpoint[pos] = 0;

			pos++;

			if( pos == searchLen )
			{
				return 0;
			}			      
		} // while (2)

refil:
		// refill x0a and x0b	
		x0a.byte[0] = list[ workpoint[0] ];
		x0a.byte[1] = list[ workpoint[1] ];
		x0a.byte[2] = list[ workpoint[2] ];
		x0a.byte[3] = list[ workpoint[3] ];

		x0b.dword = x0a.dword;
		x0b.byte[0] = list[ workpoint[0]+1 ];

	} // while (1)
  
	return 0;

}

static int
MD5TransformL7( const ord32 x0a, const ord32 x0b, const ord32 x1, const ord32 x14 )
{
  ord32	a, b, c, d;
  ord32	e, f, g, h;
  	
  a = 0x67452301;
  b = 0xefcdab89;
  c = 0x98badcfe;
  d = 0x10325476;
  
  e = 0x67452301;
  f = 0xefcdab89;
  g = 0x98badcfe;
  h = 0x10325476; 


  FF (a, b, c, d, x0a, S11, 0xd76aa478); /* 1 */  
  FF (e, f, g, h, x0b, S11, 0xd76aa478); /* 1 */  
  FF (d, a, b, c, x1, S12, 0xe8c7b756); /* 2 */
  FF (h, e, f, g, x1, S12, 0xe8c7b756); /* 2 */
  FF0 (c, d, a, b, S13, 0x242070db); /* 3 */
  FF0 (g, h, e, f, S13, 0x242070db); /* 3 */
  FF0 (b, c, d, a, S14, 0xc1bdceee); /* 4 */
  FF0 (f, g, h, e, S14, 0xc1bdceee); /* 4 */

  FF0 (a, b, c, d, S11, 0xf57c0faf); /* 5 */
  FF0 (e, f, g, h, S11, 0xf57c0faf); /* 5 */
  FF0 (d, a, b, c, S12, 0x4787c62a); /* 6 */
  FF0 (h, e, f, g, S12, 0x4787c62a); /* 6 */
  FF0 (c, d, a, b, S13, 0xa8304613); /* 7 */
  FF0 (g, h, e, f, S13, 0xa8304613); /* 7 */
  FF0 (b, c, d, a, S14, 0xfd469501); /* 8 */
  FF0 (f, g, h, e, S14, 0xfd469501); /* 8 */

  FF0 (a, b, c, d, S11, 0x698098d8); /* 9 */
  FF0 (e, f, g, h, S11, 0x698098d8); /* 9 */
  FF0 (d, a, b, c, S12, 0x8b44f7af); /* 10 */
  FF0 (h, e, f, g, S12, 0x8b44f7af); /* 10 */
  FF0 (c, d, a, b, S13, 0xffff5bb1); /* 11 */
  FF0 (g, h, e, f, S13, 0xffff5bb1); /* 11 */
  FF0 (b, c, d, a, S14, 0x895cd7be); /* 12 */
  FF0 (f, g, h, e, S14, 0x895cd7be); /* 12 */

  FF0 (a, b, c, d, S11, 0x6b901122); /* 13 */
  FF0 (e, f, g, h, S11, 0x6b901122); /* 13 */
  FF0 (d, a, b, c, S12, 0xfd987193); /* 14 */
  FF0 (h, e, f, g, S12, 0xfd987193); /* 14 */
  FF (c, d, a, b, x14,  S13, 0xa679438e); /* 15 */
  FF (g, h, e, f, x14,  S13, 0xa679438e); /* 15 */
  FF0 (b, c, d, a,       S14, 0x49b40821); /* 16 */
  FF0 (f, g, h, e,       S14, 0x49b40821); /* 16 */

  /* Round 2 */
  GG (a, b, c, d, x1, S21, 0xf61e2562); /* 17 */
  GG (e, f, g, h, x1, S21, 0xf61e2562); /* 17 */
  GG0 (d, a, b, c, S22, 0xc040b340); /* 18 */
  GG0 (h, e, f, g, S22, 0xc040b340); /* 18 */
  GG0 (c, d, a, b, S23, 0x265e5a51); /* 19 */
  GG0 (g, h, e, f, S23, 0x265e5a51); /* 19 */
  GG (b, c, d, a, x0a, S24, 0xe9b6c7aa); /* 20 */
  GG (f, g, h, e, x0b, S24, 0xe9b6c7aa); /* 20 */
  GG0 (a, b, c, d, S21, 0xd62f105d); /* 21 */
  GG0 (e, f, g, h, S21, 0xd62f105d); /* 21 */
  GG0 (d, a, b, c, S22, 0x2441453); /* 22 */
  GG0 (h, e, f, g, S22, 0x2441453); /* 22 */
  GG0 (c, d, a, b,       S23, 0xd8a1e681); /* 23 x[15] */
  GG0 (g, h, e, f,       S23, 0xd8a1e681); /* 23 x[15] */
  GG0 (b, c, d, a, S24, 0xe7d3fbc8); /* 24 */
  GG0 (f, g, h, e, S24, 0xe7d3fbc8); /* 24 */
  GG0 (a, b, c, d, S21, 0x21e1cde6); /* 25 */
  GG0 (e, f, g, h, S21, 0x21e1cde6); /* 25 */
  GG (d, a, b, c, x14, S22, 0xc33707d6); /* 26 */
  GG (h, e, f, g, x14, S22, 0xc33707d6); /* 26 */
  GG0 (c, d, a, b, S23, 0xf4d50d87); /* 27 */
  GG0 (g, h, e, f, S23, 0xf4d50d87); /* 27 */
  GG0 (b, c, d, a, S24, 0x455a14ed); /* 28 */
  GG0 (f, g, h, e, S24, 0x455a14ed); /* 28 */
  GG0 (a, b, c, d, S21, 0xa9e3e905); /* 29 */
  GG0 (e, f, g, h, S21, 0xa9e3e905); /* 29 */
  GG0 (d, a, b, c, S22, 0xfcefa3f8); /* 30 */
  GG0 (h, e, f, g, S22, 0xfcefa3f8); /* 30 */
  GG0 (c, d, a, b, S23, 0x676f02d9); /* 31 */
  GG0 (g, h, e, f, S23, 0x676f02d9); /* 31 */
  GG0 (b, c, d, a, S24, 0x8d2a4c8a); /* 32 */
  GG0 (f, g, h, e, S24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  HH0 (a, b, c, d, S31, 0xfffa3942); /* 33 */
  HH0 (e, f, g, h, S31, 0xfffa3942); /* 33 */
  HH0 (d, a, b, c, S32, 0x8771f681); /* 34 */
  HH0 (h, e, f, g, S32, 0x8771f681); /* 34 */
  HH0 (c, d, a, b, S33, 0x6d9d6122); /* 35 */
  HH0 (g, h, e, f, S33, 0x6d9d6122); /* 35 */
  HH (b, c, d, a, x14, S34, 0xfde5380c); /* 36 */
  HH (f, g, h, e, x14, S34, 0xfde5380c); /* 36 */
  HH (a, b, c, d, x1, S31, 0xa4beea44); /* 37 */
  HH (e, f, g, h, x1, S31, 0xa4beea44); /* 37 */
  HH0 (d, a, b, c, S32, 0x4bdecfa9); /* 38 */
  HH0 (h, e, f, g, S32, 0x4bdecfa9); /* 38 */
  HH0 (c, d, a, b, S33, 0xf6bb4b60); /* 39 */
  HH0 (g, h, e, f, S33, 0xf6bb4b60); /* 39 */
  HH0 (b, c, d, a, S34, 0xbebfbc70); /* 40 */
  HH0 (f, g, h, e, S34, 0xbebfbc70); /* 40 */
  HH0 (a, b, c, d, S31, 0x289b7ec6); /* 41 */
  HH0 (e, f, g, h, S31, 0x289b7ec6); /* 41 */
  HH (d, a, b, c, x0a, S32, 0xeaa127fa); /* 42 */
  HH (h, e, f, g, x0b, S32, 0xeaa127fa); /* 42 */
  HH0 (c, d, a, b, S33, 0xd4ef3085); /* 43 */
  HH0 (g, h, e, f, S33, 0xd4ef3085); /* 43 */
  HH0 (b, c, d, a, S34, 0x4881d05); /* 44 */
  HH0 (f, g, h, e, S34, 0x4881d05); /* 44 */
  HH0 (a, b, c, d, S31, 0xd9d4d039); /* 45 */
  HH0 (e, f, g, h, S31, 0xd9d4d039); /* 45 */
  
  a += x0a;
  e += x0b;
  if( (fasta ==  a) || (fasta == e) )
	  return 1;
  
  return 0;
}

void
undo64to56L7(ord32 *out, ord32 *in) 
{
      ord32 a, b, c, d;

      a = in[0];
      b = in[1];
      c = in[2];
      d = in[3];

      RII0 (b, c, d, a, S44, 0xeb86d391); /* 64 */
      RII0 (c, d, a, b, S43, 0x2ad7d2bb); /* 63 */
      RII0 (d, a, b, c, S42, 0xbd3af235); /* 62 */
      RII0 (a, b, c, d, S41, 0xf7537e82); /* 61 */
      RII0 (b, c, d, a, S44, 0x4e0811a1); /* 60 */
      RII0 (c, d, a, b, S43, 0xa3014314); /* 59 */
      RII0 (d, a, b, c, S42, 0xfe2ce6e0); /* 58 */
      RII0 (a, b, c, d, S41, 0x6fa87e4f); /* 57 */
      RII0 (b, c, d, a, S44, 0x85845dd1); /* 56 */

      out[0]=a;
      out[1]=b;
      out[2]=c;
      out[3]=d;
}