/* constants for the transformation routine.
 */
// this control the ROTATE LEFT
#define S11 7
#define SS11 25
#define S12 12
#define SS12 20
#define S13 17
#define SS13 15
#define S14 22
#define SS14 10
#define S21 5
#define SS21 27
#define S22 9
#define SS22 23
#define S23 14
#define SS23 18
#define S24 20
#define SS24 12
#define S31 4
#define SS31 28
#define S32 11
#define SS32 21
#define S33 16
#define SS33 16
#define S34 23
#define SS34 9
#define S41 6
#define SS41 26
#define S42 10
#define SS42 22
#define S43 15
#define SS43 17
#define S44 21
#define SS44 11

/* F, G, H and I are basic MD5 functions.
 */

#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z)))

/* ROTATE_LEFT rotates x left n bits.
 */

#define ROTATE_LEFT(x, n) ((x)=((x) << (n)) | ((x) >> (32-n)))

//GCC assembly for x86
//#define ROTATE_LEFT(x, n) __asm { rol x, n };

// the following MS C macro gives 21% imporvement.
#ifdef _MSC_VER
    #include    <stdlib.h>

    #undef      ROTATE_LEFT
    #undef      ROTATE_RIGHT

    #pragma intrinsic(_lrotl,_lrotr)

    #define     ROTATE_LEFT(x,n)        (x) = _lrotl((x),(n))
    #define     ROTATE_RIGHT(x,n)       (x) = _lrotr((x),(n))                     
#endif
                                

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
 * Rotation is separate from addition to prevent recomputation.
 */

#define FF(a, b, c, d, x, s1, ac) { \
 a += F ((b), (c), (d)) + (x) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define GG(a, b, c, d, x, s1, ac) { \
 (a) += G ((b), (c), (d)) + (x) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define HH(a, b, c, d, x, s1, ac) { \
 (a) += H ((b), (c), (d)) + (x) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define II(a, b, c, d, x, s1, ac) { \
 (a) += I ((b), (c), (d)) + (x) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }

#define FF0(a, b, c, d, s1, ac) { \
 (a) += F ((b), (c), (d)) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define GG0(a, b, c, d, s1, ac) { \
 (a) += G ((b), (c), (d)) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define HH0(a, b, c, d, s1, ac) { \
 (a) += H ((b), (c), (d)) + (ord32)(ac); \
 ROTATE_LEFT ((a), (s1)); \
 (a) += (b); \
  }
#define II0(a, b, c, d, s1, ac) { \
 (a) += I ((b), (c), (d)) + (ord32)(ac); \
 ROTATE_LEFTo ((a), (s1)); \
 (a) += (b); \
  }

#define RHH0(a, b, c, d, s1, ac) { \
 (a) -= (b); \
 ROTATE_RIGHT ((a), (s1)); \
 (a) -= (H ((b), (c), (d)) + (ord32)(ac)); \
  }

#define RII0(a, b, c, d, s1, ac) { \
 (a) -= (b); \
 ROTATE_LEFT ((a), (32-s1)); \
 (a) -= (I ((b), (c), (d)) + (ord32)(ac)); \
  }

#define RII(a, b, c, d, x, s1, ac) { \
 (a) -= (b); \
 ROTATE_LEFT ((a), (32-s1)); \
 (a) -= (I ((b), (c), (d)) + x + (ord32)(ac)); \
  }
